#include <hex.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>

byte *fname = NULL;
byte *text = NULL;
off_t text_size = 0;

off_t cursor = 0; char cursor_type = CURSOR_HEX;
int modified = 0;
int sigint_event = 0;

byte *search_txt = NULL;
off_t search_len;

static byte *get_string(char *prompt, off_t *len);
static void find_txt(void);
static void replace_txt(byte *txt, off_t len);

void ask_for_exit(void)
{
	int i;
	
	if (modified) {
		i = ask_status("Lose changes to this file (y,n,^C)? ");
		if (i) complain(0, "File %s not saved.\n", fname);
	} else {
		complain(0, "File %s not changed so no update needed.\n",
			fname);
		i = 1;
	}
	if (i) clean_exit();
}

void on_sigint(int x)
{
	signal(SIGINT, on_sigint);
	sigint_event = 1;
}

int main(int argc, char *argv[])
{
	time_t curtime;
	
	if (argc != 2) {
		fprintf(stderr, "Ptolomei's Hex Editor v1.0 (1999-02-02)\n");
		fprintf(stderr, "Copyright (C) 1997-2000  Senko Rasic <senko@fly.srk.fer.hr>\n");
		fprintf(stderr, "This is free software with ABSOLUTELY NO WARRANTY.\n");
		fprintf(stderr, "\nUsage: %s <filename>\n", argv[0]);
		return EXIT_FAILURE;
	}

	text = read_data(argv[1], &text_size, NULL, 0);
	if (!text) return EXIT_FAILURE;

	signal(SIGINT, on_sigint);

	if (init_screen()) {
		fprintf(stderr, "Can't initialize screen.\n");
		return EXIT_FAILURE;
	}

	fname = argv[1];
	show_top(modified);
	update_screen(1);
	print_help(1);

	cursor_type = CURSOR_HEX;

	curtime = time(NULL) / 60;
	
	sigint_event = 0;
	while (1) {
		usleep(10000);
		if ((time(NULL) / 60) > curtime) {
			show_top(modified);
			move_cursor(cursor);
			curtime = time(NULL) / 60;
		}
		main_loop();
		if (sigint_event) {
			ask_for_exit();
			sigint_event = 0;
		}
	}
	clean_exit();
	return EXIT_SUCCESS;
}

void goto_offset(void)
{
	char tmp[256];
	off_t new_curs;

	if (input_status("Go to offset (^C to abort): ", tmp, 255))
		return;
	new_curs = strtoul(tmp, NULL, 0);
	if (new_curs >= text_size) new_curs = text_size - 1;
	move_cursor(new_curs);
}

static byte *get_string(char *prompt, off_t *len)
{
	byte *buf;
	char tmp[256];
	int i = 0;

	i = input_status(prompt, tmp, 255);
	
	move_cursor(cursor);
	if (i) return NULL;

	if (cursor_type == CURSOR_HEX) {
		char tmp2[256];
		char *a = tmp, *b = tmp;
		i = 0;
		while (*a) {
			tmp2[i] = (byte) strtol(a, (char **) &a, 16);
			if (a == b) return NULL;
			b = a;
			i++;
		}
		tmp2[i] = 0;
		*len = i;
		buf = malloc(i);
		if (!buf) return NULL;
		memcpy(buf, tmp2, i);
	} else {
		*len = strlen(tmp);
		buf = malloc(*len + 1);
		if (!buf) return NULL;
		strncpy(buf, tmp, *len);
		buf[*len] = 0;
	}
	return buf;
}

static void find_txt(void)
{
	off_t nc;
		
	if (!search_txt) return;
	nc = bfi_search();
	if (nc == cursor) {
		complain(0, "Not found");
		move_cursor(cursor);
	} else {
		move_cursor(nc);
	}
}

static void replace_txt(byte *txt, off_t len)
{
	int i;
	off_t nc, oc;
	
	i = 0;
	oc = cursor;
	while (1) {
		nc = bfi_search();
		if (nc == oc) {
			if (!i)
				complain(0, "Not found");
			else {
				complain(0, "Made %d substitutions", i);
				modified = 1; show_top(modified);
			}
			update_screen(0);
			move_cursor(nc);
			break;
		}
		cursor = nc;
		oc = nc - search_len;
		if ((oc + len) > text_size) len = text_size - oc;
		memcpy(text + oc, txt, len);
		oc = nc;
		i++;
	}
}

void snr_txt(void)
{
	char tmp[256];
	char *txt = NULL;
	off_t len;
		
	if (search_txt) free(search_txt);

	if (cursor_type == CURSOR_HEX)
		search_txt = get_string("Find hex (^C to abort): ", &search_len);
	else
		search_txt = get_string("Find text (^C to abort): ", &search_len);
	if (!search_txt) return;

	tmp[0] = 0;
	if (input_status("(S)earch (R)eplace (^C to abort): ", tmp, 255))
		return;

	if (!*tmp || (toupper(*tmp) == 'S')) {
		find_txt();
		return;
	}

	if (cursor_type == CURSOR_HEX)
		txt = get_string("Replace with hex (^C to abort): ", &len);
	else
		txt = get_string("Replace with text (^C to abort): ", &len);

	if (!txt) return;

	replace_txt(txt, len);
	free(txt);
}

/* very slow (and simple) BruteForceAndIgnorance search */
off_t bfi_search(void)
{
	off_t i;
	
	if (!search_txt) return cursor;

	for (i = cursor + search_len; i < text_size; i++) {
		if (!memcmp(text + i - search_len, search_txt, search_len))
			return i;
	}
	return cursor;
}

void copy_block(off_t s, off_t e)
{
	byte *a;
	off_t i;

	if (s >= text_size) s = text_size;
	if (e >= text_size) e = text_size;
	if (s > e) return;
	
	i = e - s + 1;
	a = (byte *) malloc(i);
	if (!a) return;
	
	memcpy(a, text + s, i);
	if ((cursor + i) > text_size) i = text_size - cursor;
	memcpy(text + cursor, a, i);
	modified = 1;
	show_top(modified);
	update_screen(0);
	move_cursor(cursor);
}

